﻿using Microsoft.AspNetCore.Mvc;
using TECIT.TBarCode;
using System.Drawing;
using static System.Net.Mime.MediaTypeNames;
using System.Runtime.Versioning;
using System.Drawing.Imaging;

namespace ASPNETCoreWebApp
{
    [SupportedOSPlatform("windows")]
    /// <summary>
    /// Wrapper class for Non GDI Callback Drawing
    /// </summary>
    public class BarcodeNoGDI
    {
        private Graphics? gfx = null;                     // Current graphics object for painting.

        private Color BarcodeColor { get; set; }
        private Color BackgroundColor { get; set; }
        private Color FontColor { get; set; }
        private int BarcodeWidthPx { get; set; }
        private int BarcodeHeightPx { get; set; }
        private int BarcodeRotation { get; set; }
        private BarShape BarcodeBarShape { get; set; }
        private BarcodeType BarcodeType { get; set; }

        private Barcode _barcode;

        /// <summary>
        /// Create wrapper for Non GDI Callback Drawing
        /// </summary>
        /// <param name="barcode"></param>
        /// <exception cref="ArgumentNullException"></exception>
        public BarcodeNoGDI (Barcode barcode) {
            
            if (barcode == null)
                throw new ArgumentNullException(nameof(barcode));

            _barcode = barcode;

            _barcode.DrawText += OnDrawText;
            _barcode.DrawBar += OnDrawBar;
            _barcode.DrawColoredBar += OnDrawColoredBar;
            _barcode.DrawHexagon += OnDrawHexagon;
            _barcode.DrawRing += OnDrawRing;

            // Callback drawing needs a licensed bar code object
            // License at startup of your app (key stays in memory) or one time per bar code instance as shown here

            //_barcode.License("<your licensee name>", LicenseType.DeveloperOrWeb, 1, "<your key>", TBarCodeProduct.Barcode2D);

            // You can also license via barcode.ini file (post compile time)
            // You can request a free demo license from our sales@tec-it.com
        }

        /// <summary>
        /// Generates the barcode.
        /// </summary>
        public void GenerateAsBitmap(System.IO.Stream stream, ImageFormat format)
        {
            // This does not work in Azure
            //using (Bitmap barcodeImage = barcode.DrawBitmap (barcode.BoundingRectangle.Width, barcode.BoundingRectangle.Height))
            //{
            //  barcodeImage.SetResolution ((float) barcode.Dpi, (float) barcode.Dpi);
            //  barcodeImage.Save (stream, ImageFormat);
            //}

            // instead generate barcode using callbacks

            BackgroundColor = _barcode.BackgroundColor;
            BarcodeColor = _barcode.BarcodeColor;
            FontColor = _barcode.FontColor;
            BarcodeWidthPx = _barcode.BoundingRectangle.Width;
            BarcodeHeightPx = _barcode.BoundingRectangle.Height;
            BarcodeRotation = (int)_barcode.Rotation * 90;
            BarcodeBarShape = _barcode.BarShape;
            BarcodeType = _barcode.BarcodeType;

            using (Bitmap barcodeImage = new Bitmap(BarcodeWidthPx, BarcodeHeightPx))
            {
                gfx = Graphics.FromImage(barcodeImage);

                if (!_barcode.IsBackgroundTransparent)
                {
                    using (Brush brush = new SolidBrush(BackgroundColor))
                    {
                        gfx.FillRectangle(brush, 0, 0, BarcodeWidthPx, BarcodeHeightPx);
                    }
                }

                _barcode.DrawCustom(PixelToMM(_barcode.BoundingRectangle.Width, gfx.DpiX), PixelToMM(_barcode.BoundingRectangle.Height, gfx.DpiX));

                if (_barcode.Dpi < 0)
                    _barcode.Dpi = gfx.DpiX;

                barcodeImage.SetResolution((float)_barcode.Dpi, (float)_barcode.Dpi);

                barcodeImage.Save(stream, format);
            }
        }

        /// <summary>
        /// This method is called once by DrawCustom for each bar of the barcode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnDrawBar(object? sender, CustomDrawingEventArgs e)
        {
            // Get position and extent. Values have to be converted from millimeters to pixels.
            var xLeft = MMToPixel(e.X, gfx.DpiX);
            var xRight = MMToPixel(e.X + e.Width, gfx.DpiX);
            var yTop = MMToPixel(e.Y, gfx.DpiY);
            var yBottom = MMToPixel(e.Y + e.Height, gfx.DpiY);
            var width = xRight - xLeft;
            var height = yBottom - yTop;

            using (Brush brush = new SolidBrush(BarcodeColor))
            {
                // would be nice, but we get BarShape.Default even for DotCode - maybe in future tbarcode versions
                //switch(BarcodeBarShape)
                //{
                //  case BarShape.Ellipse:
                //    gfx.FillEllipse(brush, x, y, width, height);
                //    break;
                //  default:
                //    // Draw bar.
                //    gfx.FillRectangle(brush, x, y, width, height);
                //    break;
                //}

                switch (BarcodeType)
                {
                    case BarcodeType.DotCode:
                        // Draw Circle.
                        gfx.FillEllipse(brush, xLeft, yTop, width, height);
                        break;
                    default:
                        // Draw bar.
                        gfx.FillRectangle(brush, xLeft, yTop, width, height);
                        break;
                }
            }
        }

        public void OnDrawColoredBar(object? sender, CustomDrawingEventArgs e)
        {
            // Get position and extent. Values have to be converted from millimeters to pixels.
            var xLeft = MMToPixel(e.X, gfx.DpiX);
            var xRight = MMToPixel(e.X + e.Width, gfx.DpiX);
            var yTop = MMToPixel(e.Y, gfx.DpiY);
            var yBottom = MMToPixel(e.Y + e.Height, gfx.DpiY);
            var width = xRight - xLeft;
            var height = yBottom - yTop;

            using (Brush brush = new SolidBrush(e.Color ?? BarcodeColor))
            {
                gfx.FillRectangle(brush, xLeft, yTop, width, height);
            }
        }

        /// <summary>
        /// This method is called by DrawCustom for drawing the human readable text.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnDrawText(object? sender, CustomDrawingEventArgs e)
        {
            // Make sure that we have a font.
            if (e.Font == null)
                e.Font = new Font("Arial", 14);

            // Set alignment - use StringFormat.GenericTypographic to remove extra padding in left/right aligned text
            using (StringFormat stringFormat = StringFormat.GenericTypographic)
            {
                float posX = MMToPixel(e.X, gfx.DpiX);
                float posY = MMToPixel(e.Y, gfx.DpiY);

                if (e.Alignment == Alignment.Left)
                {
                    stringFormat.Alignment = StringAlignment.Near;
                }
                else if (e.Alignment == Alignment.Right)
                {
                    stringFormat.Alignment = StringAlignment.Far;
                }
                else
                {
                    stringFormat.Alignment = StringAlignment.Center;
                }

                // Optional: Fix font spacing for ">" in EAN-13
                // if (e.Text == ">")
                //  e.X = e.X + e.Font.SizeInPoints / 14;

                using (Brush brush = new SolidBrush(FontColor))
                {
                    // Set text rotation
                    gfx.TranslateTransform(posX, posY);
                    gfx.RotateTransform(-BarcodeRotation);

                    // Draw text
                    gfx.DrawString(e.Text, e.Font, brush, 0, 0, stringFormat);

                    gfx.ResetTransform();
                }
            }

            // Alternative - draw with font spacing
            /*
            // given in Point
            float characterDistance = 1.0F;

            // convert to Pixel (1 Pt = 1/72 inch)
            characterDistance = characterDistance * gfx.DpiX / 72.0F;

            float[] charPositionsX = new float[e.Text.Length];
            float textWidth = 0;

            for (int i = 0; i < e.Text.Length; i++)
            {
              SizeF size = gfx.MeasureString(e.Text[i].ToString(), e.Font, 0, StringFormat.GenericTypographic);
              textWidth += size.Width + characterDistance;
              charPositionsX[i] = textWidth;
            }

            // adjust bar code position for centered text
            if (stringFormat.Alignment == StringAlignment.Center)
            {
              posX = posX - (textWidth / 2);
              stringFormat.Alignment = StringAlignment.Near;
            }

            // Note: Right alignment needs extra handling

            // now draw each character separately
            float charPosX = posX; // start pos is left

            for (int i = 0; i < e.Text.Length; i++)
            {
              gfx.DrawString(e.Text[i].ToString(), e.Font, Brushes.Blue, charPosX, posY, stringFormat);
              charPosX = posX + charPositionsX[i];
            }
            */
        }

        /// <summary>
        /// This method is called once by DrawCustom for each Hexagon of the barcode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnDrawHexagon(object? sender, CustomDrawingEventArgs e)
        {
            Point[] hexagonPoints = new Point[6];

            int i = 0;
            for (i = 0; i < 6; i++)
            {
                hexagonPoints[i].X = MMToPixel(e.Points_x[i], gfx.DpiX);
                hexagonPoints[i].Y = MMToPixel(e.Points_y[i], gfx.DpiY);
            }

            using (Brush brush = new SolidBrush(BarcodeColor))
            {
                using (Pen pen = new Pen(BarcodeColor))
                {
                    // Draw polygon to screen.
                    gfx.DrawPolygon(pen, hexagonPoints);
                    gfx.FillPolygon(brush, hexagonPoints, System.Drawing.Drawing2D.FillMode.Alternate);

                }
            }
        }

        /// <summary>
        /// This method is called once by DrawCustom for each Hexagon of the barcode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnDrawRing(object? sender, CustomDrawingEventArgs e)
        {
            double thickness = e.RadiusOuter - e.RadiusInner;
            double diameter = 2 * (e.RadiusInner + (thickness / 2));
            int widthPx = MMToPixel(diameter, gfx.DpiX);
            int heightPx = MMToPixel(diameter, gfx.DpiY);
            int posX = MMToPixel(e.X - (diameter / 2), gfx.DpiX);
            int posY = MMToPixel(e.Y - (diameter / 2), gfx.DpiX);

            using (Pen pen = new Pen(BarcodeColor, MMToPixel(thickness, gfx.DpiX)))
            {
                gfx.DrawArc(pen, posX, posY, widthPx, heightPx, 0, 360);
            }
        }

        // Helper function: Converts x from millimeters to pixels.
        private int MMToPixel(double x, double dpi)
        {
            return (int)(0.5 + x / 25.4 * dpi);
        }

        // Helper function: Converts x from millimeters to pixels.
        private float PixelToMM(float x, float dpi)
        {
            return (float)(x * 25.4 / dpi);
        }
    }
}
